[clang] 3cec39b - [C++20] [Modules] Don't generate strong function of a partition in importing modules

Chuanqi Xu via cfe-commits cfe-commits at lists.llvm.org
Thu Mar 31 23:36:51 PDT 2022


Author: Chuanqi Xu
Date: 2022-04-01T14:36:04+08:00
New Revision: 3cec39b91c50944095affc52eee286e3161772b4

URL: https://github.com/llvm/llvm-project/commit/3cec39b91c50944095affc52eee286e3161772b4
DIFF: https://github.com/llvm/llvm-project/commit/3cec39b91c50944095affc52eee286e3161772b4.diff

LOG: [C++20] [Modules] Don't generate strong function of a partition in importing modules

This solves the multiple defintiion issue
https://github.com/llvm/llvm-project/issues/54269.

I think the cause of the bug might be an oversight. We just forget to
edit this when implementing partitions. And it should be a good fix.

Reviewed By: iains

Differential Revision: https://reviews.llvm.org/D121271

Added: 
    clang/test/CodeGenCXX/partitions.cpp

Modified: 
    clang/include/clang/Basic/Module.h
    clang/lib/Serialization/ASTWriterDecl.cpp

Removed: 
    


################################################################################
diff  --git a/clang/include/clang/Basic/Module.h b/clang/include/clang/Basic/Module.h
index 4500c78ec6824..af96038ba4aa8 100644
--- a/clang/include/clang/Basic/Module.h
+++ b/clang/include/clang/Basic/Module.h
@@ -529,6 +529,10 @@ class Module {
 
   /// Is this module a header unit.
   bool isHeaderUnit() const { return Kind == ModuleHeaderUnit; }
+  // Is this a C++20 module interface or a partition.
+  bool isInterfaceOrPartition() const {
+    return Kind == ModuleInterfaceUnit || isModulePartition();
+  }
 
   /// Get the primary module interface name from a partition.
   StringRef getPrimaryModuleInterfaceName() const {

diff  --git a/clang/test/CodeGenCXX/partitions.cpp b/clang/test/CodeGenCXX/partitions.cpp
new file mode 100644
index 0000000000000..865af5f161eff
--- /dev/null
+++ b/clang/test/CodeGenCXX/partitions.cpp
@@ -0,0 +1,50 @@
+// RUN: rm -rf %t
+// RUN: mkdir %t
+// RUN: split-file %s %t
+
+// RUN: %clang_cc1 -std=c++20 -emit-module-interface -triple %itanium_abi_triple %t/parta.cppm -o %t/mod-parta.pcm
+// RUN: %clang_cc1 -std=c++20 -emit-module-interface -triple %itanium_abi_triple %t/partb.cppm -o %t/mod-partb.pcm
+// RUN: %clang_cc1 -std=c++20 -emit-module-interface -triple %itanium_abi_triple -fmodule-file=%t/mod-parta.pcm \
+// RUN:     -fmodule-file=%t/mod-partb.pcm %t/mod.cppm -o %t/mod.pcm
+// RUN: %clang_cc1 -std=c++20 -triple %itanium_abi_triple %t/mod.pcm -S -emit-llvm -disable-llvm-passes -o - \
+// RUN:     | FileCheck %t/mod.cppm
+// RUN: %clang_cc1 -std=c++20 -O2 -emit-module-interface -triple %itanium_abi_triple -fmodule-file=%t/mod-parta.pcm \
+// RUN:     -fmodule-file=%t/mod-partb.pcm %t/mod.cppm -o %t/mod.pcm
+// RUN: %clang_cc1 -std=c++20 -O2 -triple %itanium_abi_triple %t/mod.pcm -S -emit-llvm -disable-llvm-passes -o - \
+// RUN:     | FileCheck %t/mod.cppm  -check-prefix=CHECK-OPT
+
+//--- parta.cppm
+export module mod:parta;
+
+export int a = 43;
+
+export int foo() {
+  return 3 + a;
+}
+
+//--- partb.cppm
+module mod:partb;
+
+int b = 43;
+
+int bar() {
+  return 43 + b;
+}
+
+//--- mod.cppm
+export module mod;
+import :parta;
+import :partb;
+export int use() {
+  return foo() + bar() + a + b;
+}
+
+// CHECK: @_ZW3mod1a = available_externally global
+// CHECK: @_ZW3mod1b = available_externally global
+// CHECK: declare{{.*}} i32 @_ZW3mod3foov
+// CHECK: declare{{.*}} i32 @_ZW3mod3barv
+
+// CHECK-OPT: @_ZW3mod1a = available_externally global
+// CHECK-OPT: @_ZW3mod1b = available_externally global
+// CHECK-OPT: define available_externally{{.*}} i32 @_ZW3mod3foov
+// CHECK-OPT: define available_externally{{.*}} i32 @_ZW3mod3barv


        


More information about the cfe-commits mailing list
diff  --git a/clang/lib/Serialization/ASTWriterDecl.cpp b/clang/lib/Serialization/ASTWriterDecl.cpp
index 39ad96562c92c..4e9e97514de0b 100644
--- a/clang/lib/Serialization/ASTWriterDecl.cpp
+++ b/clang/lib/Serialization/ASTWriterDecl.cpp
@@ -1029,12 +1029,12 @@ void ASTDeclWriter::VisitVarDecl(VarDecl *D) {
     if (Writer.WritingModule &&
         !D->getDescribedVarTemplate() && !D->getMemberSpecializationInfo() &&
         !isa<VarTemplateSpecializationDecl>(D)) {
       // When building a C++ Modules TS module interface unit, a strong
       // definition in the module interface is provided by the compilation of
       // that module interface unit, not by its users. (Inline variables are
       // still emitted in module users.)
       ModulesCodegen =
-          (Writer.WritingModule->Kind == Module::ModuleInterfaceUnit ||
+          (Writer.WritingModule->isInterfaceOrPartition() ||
            (D->hasAttr<DLLExportAttr>() &&
             Writer.Context->getLangOpts().BuildingPCHWithObjectFile)) &&
           Writer.Context->GetGVALinkageForVariable(D) == GVA_StrongExternal;
@@ -2470,11 +2470,11 @@ void ASTRecordWriter::AddFunctionDefinition(const FunctionDecl *FD) {
   if (!FD->isDependentContext()) {
     Optional<GVALinkage> Linkage;
     if (Writer->WritingModule &&
-        Writer->WritingModule->Kind == Module::ModuleInterfaceUnit) {
-      // When building a C++ Modules TS module interface unit, a strong
-      // definition in the module interface is provided by the compilation of
-      // that module interface unit, not by its users. (Inline functions are
-      // still emitted in module users.)
+        Writer->WritingModule->isInterfaceOrPartition()) {
+      // When building a C++20 module interface unit or a partition unit, a
+      // strong definition in the module interface is provided by the
+      // compilation of that unit, not by its users. (Inline functions are still
+      // emitted in module users.)
       Linkage = Writer->Context->GetGVALinkageForFunction(FD);
       ModulesCodegen = *Linkage == GVA_StrongExternal;
     }
@@ -1029,12 +1029,12 @@ void ASTDeclWriter::VisitVarDecl(VarDecl *D) {
         !D->getDescribedVarTemplate() && !D->getMemberSpecializationInfo() &&
         !isa<VarTemplateSpecializationDecl>(D)) {
-      // When building a C++ Modules TS module interface unit, a strong
-      // definition in the module interface is provided by the compilation of
-      // that module interface unit, not by its users. (Inline variables are
-      // still emitted in module users.)
+      // When building a C++20 module interface unit or a partition unit, a
+      // strong definition in the module interface is provided by the
+      // compilation of that unit, not by its users. (Inline variables are still
+      // emitted in module users.)
       ModulesCodegen =
